import Houk from 'houk';

interface CocyOptions {
    /**
     * Glob patterns for files
     * @name patterns
     * @default "['*.md', '*.markdown', '*.mdwn', '!.*', '!_*']"
     */
    patterns: string[];
    /**
     * The current working directory.
     * @name cwd
     * @default process.cwd()
     */
    cwd: string;
    /**
     * Watch files and rebuild on change
     * @name watch
     * @default "process.env.NODE_ENV === 'development'"
     */
    watch: boolean;
    /**
     * A function that slugifies a given input.
     * @name slugify
     * @default slugo
     */
    slugify: (input: string) => string;
}

declare type CocyFileAttributes = {
    [key: string]: any;
    createdAt?: Date;
    modifiedAt?: Date;
};
declare class CocyFile<DataType = any> extends Houk<{
    /**
     * File has been read and content saved to `raw`.
     */
    fileRead: [];
}> {
    private instance;
    /**
     * Absolute path to the file
     */
    path: string;
    /**
     * Mime type of the file
     */
    mimeType: string | false;
    /**
     * Slug of the file
     * @name slug
     */
    slug: string;
    /**
     * File contents before any transformation
     */
    raw?: string;
    /**
     * Transformed file contents
     * @name data
     */
    data?: DataType;
    /**
     * File attributes
     * @name attributes
     */
    attributes: CocyFileAttributes;
    /**
     * Resolved assets.
     */
    assets: Map<string, string>;
    constructor(instance: Cocy, path: string);
    load(): void;
    /**
     * Resolve an asset.
     * @param assetPath The absolute path of the asset.
     * @param key An optional key, with which the asset can be retrieved from the file.
     */
    resolveAsset(assetPath: string, key?: string): Promise<string>;
    /**
     * Generates a slug and updates the file.
     * @param text The text to slugify.
     * @param slugify Slugify the text using the set slugify function. True by default.
     */
    setSlug(text: string, slugify?: boolean): string;
}

declare type CocyEvents = {
    /**
     * Triggered when a new file was added.
     */
    fileAdded: [file: CocyFile];
    /**
     * Triggered when a file was removed.
     */
    fileRemoved: [file: CocyFile];
    /**
     * Triggered when a file's content changed.
     */
    fileUpdated: [file: CocyFile];
    /**
     * Triggered when any file event occured (added, deleted...)
     */
    fileChanged: [file: CocyFile];
    /**
     * Triggered when a new asset was found.
     */
    assetAdded: [
        resolve: (resolvedPath: string) => void,
        assetPath: string,
        file: CocyFile,
        key?: string
    ];
};

declare class CocyFiles extends Map<string, CocyFile> {
    private instance;
    constructor(instance: Cocy, files?: Iterable<[string, CocyFile]>);
    private toArray;
    /**
     * Find a file using Array.find
     * @param fn Iterator function
     * @returns file, if found
     */
    find(fn: (file: CocyFile, index: number) => boolean): CocyFile | undefined;
    /**
     * Filter files using Array.filter
     * @param fn Iterator function
     * @returns filtered files
     */
    filter(fn: (file: CocyFile, index: number) => boolean): CocyFile[];
    /**
     * get a file by its slug
     * @param slug Slug to find
     * @returns file, if found
     */
    getBySlug(slug: string): CocyFile | undefined;
    /**
     * get a file by its relative or absolute path
     * @param segments path segments passed to path.join
     * @returns file, if found
     */
    getByPath(...segments: string[]): CocyFile | undefined;
    /**
     * get all files that are located in a given directory
     * @param _dir relative or absolute path to dir
     * @returns files
     */
    getInDir(_dir: string): CocyFile[];
}

declare type DropFirstInTuple<T extends any[]> = T extends [arg: any, ...rest: infer U] ? U : T;
declare class Cocy extends Houk<CocyEvents> {
    options: CocyOptions;
    files: CocyFiles;
    isGitRepo: boolean;
    private watcher?;
    slugs: Set<string>;
    getListeners: <EventName extends keyof CocyEvents>(event: EventName) => {
        fileAdded: Set<(file: CocyFile<any>) => void>;
        fileRemoved: Set<(file: CocyFile<any>) => void>;
        fileUpdated: Set<(file: CocyFile<any>) => void>;
        fileChanged: Set<(file: CocyFile<any>) => void>;
        assetAdded: Set<(resolve: (resolvedPath: string) => void, assetPath: string, file: CocyFile<any>, key?: string | undefined) => void>;
    }[EventName];
    emit: <EventName extends keyof CocyEvents>(event: EventName, ...args: CocyEvents[EventName]) => Promise<void>;
    constructor(options?: Partial<CocyOptions>);
    /**
     * Search for files
     * @param cwd Directory to search files in
     * @default cwd Cocy instance cwd
     */
    discover(cwd?: string): Promise<Cocy>;
    /**
     * Add or update file
     * @param filepath: Absolute path to file
     */
    add(filepath: string): Promise<void>;
    /**
     * Remove file
     * @param _file File to remove. Either a CocyFile or an absolute path
     * @returns True on success, false if file didn't exist
     */
    remove(_file: CocyFile | string): boolean;
    /**
     * Update a file.
     * @param file
     */
    update(file: CocyFile): void;
    /**
     * Recursively deletes all files in a given directory.
     * @param dirpath Absolute path of the directory to remove.
     */
    removeDir(dirpath: string): void;
    /**
     * Start watching the file system for changes.
     */
    startWatcher(): void;
    /**
     * Stop watching the file system for changes.
     */
    stopWatcher(): void;
    /**
     * Call a plugin function, so it can register hooks.
     * @param plugin The plugin which will be registered.
     * @param options Options, which will be passed to the plugin.
     */
    use<Plugin extends (instance: Cocy, ...args: any) => void>(plugin: Plugin, ...options: DropFirstInTuple<Parameters<Plugin>>): this;
}

export default Cocy;
export { CocyFile, CocyOptions };
